#pragma once
#include "function_header.h"
#include <functional>
#include <limits>

namespace NJS::Class
{
	// Constructors
	Function::Function(){}
	Function::Function(void *val)
	{
		counter++;
		value = (NJS::Type::function_t*)val;
	}
	Function::Function(void *val, NJS::VAR bind)
	{
		counter++;
		This = bind;
		value = (NJS::Type::function_t*)val;
	}
	// Methods
	inline void Function::Delete() noexcept
	{
		if (--counter == 0)
		{
			delete (NJS::Type::function_t*)value;
			delete this;
		}
	}
	inline void Function::jsDelete(const std::string _key) noexcept
	{
		#ifdef __NJS__OBJECT_HASHMAP
			object.erase(_key);
		#else
			for (NJS::Type::object_t::iterator it = object.begin() ; it != object.end(); ++it)
			{
				if (_key.compare(it->first) == 0)
				{
					object.erase(it);
					return;
				}
			}
		#endif
	}
	inline void* Function::Copy() noexcept
	{
		counter++;
		return this;
	}
	// Native cast
	Function::operator bool() const noexcept { return true; }
	Function::operator double() const noexcept
	{
		return std::numeric_limits<double>::quiet_NaN();
	}
	Function::operator int() const noexcept
	{
		return std::numeric_limits<int>::quiet_NaN();
	}
	Function::operator long long() const noexcept
	{
		return std::numeric_limits<long long>::quiet_NaN();
	}
	Function::operator std::string() const noexcept
	{
		#ifdef __NJS_DEBUG
		return code;
		#else 
		return "[native code]";
		#endif
	}
	// Main operators
	NJS::VAR const Function::operator[](NJS::VAR key) const
	{
		return NJS::Global::undefined;
	}
	
	#ifdef __NJS__OBJECT_HASHMAP
	NJS::VAR &Function::operator[](NJS::VAR key)
	{
		std::string _str = ((std::string)key);
		NJS::Type::StringView _sview = _str;
		
		NJS::VAR& _obj = object[_str];
		if(_obj.type != NJS::Enum::Type::Undefined) return _obj; 		
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		return _obj;
	}
	#else
	NJS::VAR &Function::operator[](NJS::VAR key)
	{
		std::string _str = ((std::string)key);
		NJS::Type::StringView _sview = _str;
		
		for (auto & search : object)
		{
			if (_sview.compare(search.first) == 0)
			{
				return search.second;
			}
		}
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		object.push_back(NJS::Type::pair_t(_str, NJS::Global::undefined));
		return object[object.size() - 1].second;
	}
	#endif
	
	#ifdef __NJS__OBJECT_HASHMAP
	NJS::VAR &Function::operator[](int key)
	{
		std::string _str = std::to_string(key);
		NJS::Type::StringView _sview = _str;
		
		NJS::VAR& _obj = object[_str];
		if(_obj.type != NJS::Enum::Type::Undefined) return _obj; 		
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		return _obj;
	}
	#else
	NJS::VAR &Function::operator[](int key)
	{
		std::string _str = std::to_string(key);
		NJS::Type::StringView _sview = _str;
		
		for (auto & search : object)
		{
			if (_sview.compare(search.first) == 0)
			{
				return search.second;
			}
		}
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		object.push_back(NJS::Type::pair_t(_str, NJS::Global::undefined));
		return object[object.size() - 1].second;
	}
	#endif
	
	#ifdef __NJS__OBJECT_HASHMAP
	NJS::VAR &Function::operator[](double key)
	{
		std::string _str = std::to_string(key);
		NJS::Type::StringView _sview = _str;
		
		NJS::VAR& _obj = object[_str];
		if(_obj.type != NJS::Enum::Type::Undefined) return _obj; 		
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		return _obj;
	}
	#else
	NJS::VAR &Function::operator[](double key)
	{
		std::string _str = std::to_string(key);
		NJS::Type::StringView _sview = _str;
		
		for (auto & search : object)
		{
			if (_sview.compare(search.first) == 0)
			{
				return search.second;
			}
		}
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		object.push_back(NJS::Type::pair_t(_str, NJS::Global::undefined));
		return object[object.size() - 1].second;
	}
	#endif
	
	#ifdef __NJS__OBJECT_HASHMAP
	NJS::VAR &Function::operator[](const char* key)
	{
		std::string _str = key;
		NJS::Type::StringView _sview = _str;
		
		NJS::VAR& _obj = object[_str];
		if(_obj.type != NJS::Enum::Type::Undefined) return _obj; 		
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		return _obj;
	}
	#else
	NJS::VAR &Function::operator[](const char* key)
	{
		std::string _str = key;
		NJS::Type::StringView _sview = _str;
		
		for (auto & search : object)
		{
			if (_sview.compare(search.first) == 0)
			{
				return search.second;
			}
		}
		
		__NJS_Object_Lazy_Loader("prototype");
		
		__NJS_Method_Lazy_Loader("toString", toString);
		__NJS_Method_Lazy_Loader("valueOf", valueOf);
		__NJS_Method_Lazy_Loader("bind", bind);
		__NJS_Method_Lazy_Loader("call", call);
		__NJS_Method_Lazy_Loader("apply", apply);
		
		object.push_back(NJS::Type::pair_t(_str, NJS::Global::undefined));
		return object[object.size() - 1].second;
	}
	#endif
	
	inline NJS::VAR Function::Call(NJS::VAR& __NJS_THIS, NJS::VAR* _args, int i)
	{
		return (*static_cast<NJS::Type::function_t *>(value))(__NJS_THIS, _args, i);
	}
	
	
	template <class... Args>
	NJS::VAR Function::New(Args... args)
	{
		NJS::VAR _args[] = {args...};
		int i = sizeof...(args);
		
		NJS::VAR _this = __NJS_Object_Clone((*this)["prototype"]);
		if(_this.type == NJS::Enum::Type::Undefined) _this = __NJS_Create_Object();
		
		NJS::VAR _ret = this->Call(_this, _args, i);

		if(_ret.type == NJS::Enum::Type::Object)
		{
			((NJS::Class::Object*)_ret.data.ptr)->property.set(1,1);
			((NJS::Class::Object*)_ret.data.ptr)->instance.push_back((*this)["prototype"].data.ptr);
			return _ret;
		}
		else
		{
			((NJS::Class::Object*)_this.data.ptr)->property.set(1,1);
			((NJS::Class::Object*)_this.data.ptr)->instance.push_back((*this)["prototype"].data.ptr);
			return _this;
		}

	}
	
	template <class... Args>
	NJS::VAR Function::operator()(Args... args)
	{
		NJS::VAR _args[] = {args...};
		int i = sizeof...(args);
		return (*static_cast<NJS::Type::function_t *>(value))(This, _args, i);
	}
	// Comparation operators
	NJS::VAR Function::operator!() const 
	{
		return __NJS_Boolean_FALSE;
	}
	bool Function::operator==(const Function &_v1) const { return false; }
	// === emulated with __NJS_EQUAL_VALUE_AND_TYPE
	// !== emulated with __NJS_NOT_EQUAL_VALUE_AND_TYPE
	bool Function::operator!=(const Function &_v1) const { return true; }
	bool Function::operator<(const Function &_v1) const { return false; }
	bool Function::operator<=(const Function &_v1) const { return true; }
	bool Function::operator>(const Function &_v1) const { return false; }
	bool Function::operator>=(const Function &_v1) const { return true; }
	// Numeric operators
	Function Function::operator+() const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator-() const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator++(const int _v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator--(const int _v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator+(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator+=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator-(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator-=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator*(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator*=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	// TODO: "**" and "**=" operators
	Function Function::operator/(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator/=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator%(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator%=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator&(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator|(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator^(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator~() const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator>>(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator<<(const Function &_v1) const 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator&=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator|=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator^=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator>>=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	Function Function::operator<<=(const Function &_v1) 
	{
		#if !defined(__NJS_ENV_ARDUINO) && !defined(__NJS_ENV_ESP32)
		throw InvalidTypeException();
		#endif
		return Function();
	}
	// TODO: ">>>" and ">>>=" operators
	NJS::VAR Function::toString(NJS::VAR* _args, int _length) const
	{
		return (std::string)*this;
	}
	
	NJS::VAR Function::valueOf(NJS::VAR* _args, int _length) const
	{
		// TODO return this
		return NJS::Global::undefined;
	}
	NJS::VAR Function::bind(NJS::VAR* _args, int _length)
	{
		return __NJS_Create_Var_Scoped_Anon(
		counter++;
		NJS::VAR _bind;
		if(_length > 0)
		{
			_bind = _args[0];
		}
		NJS::VAR _binded = new NJS::Class::Function(value, _bind);
		return _binded;
		);
	}
	
	NJS::VAR Function::call(NJS::VAR* _args, int _length)
	{
		NJS::VAR __THIS;
		if(_length > 0)
		{
			__THIS = _args[0];
			_length--;
		}
		NJS::VAR _newArgs[_length];
		for(int i = 1; i < _length; i++) _newArgs[i-1] = _args[i];
		
		return Call(__THIS, _newArgs, _length);
	}
	
	NJS::VAR Function::apply(NJS::VAR* _args, int _length)
	{
		NJS::VAR __THIS;
		if(_length > 0)
		{
			__THIS = _args[0];
			_length--;
		}
		
		if(_length > 0)
		{
			NJS::VAR _arr = _args[1];
			if(_arr.type == NJS::Enum::Type::Array)
			{
				NJS::Type::vector_t _v = ((NJS::Class::Array*)_arr.data.ptr)->value;
				NJS::VAR __ARGS[_v.size()];
				int i = 0;
				for( auto _var: _v)
				{
					__ARGS[i] = _var;
					i++;
				}
				return Call(__THIS, __ARGS, i);
			}
		}
		
		NJS::VAR __ARGS[0];
		return Call(__THIS, __ARGS, 0);
	}
} // namespace NJS::Class
